import 'crypto.dart';
import 'codec.dart';
import 'packet.dart';
import 'utils.dart';

class Input {
  final List<int> from;
  final DateTime sentAt;
  final List<List<int>> content;
  final int kind;

  Input(this.from, this.kind, this.content, this.sentAt);

  static Future<Input> decrypt(Packet packet, List<int> sk) async {
    var key = await Crypto.deriveKey(sk, packet.from);
    var data = await Crypto.decrypt(key, packet.data);
    var args = Codec.decode(data);
    var time =
        DateTime.fromMicrosecondsSinceEpoch(Utils.parseBinaryToInt(args[0]));
    return Input(packet.from, args[1][0], Codec.decode(args[2]), time);
  }
}

class Output {
  final List<int> to;
  late final DateTime sentAt;
  final List<List<int>> content;
  final int kind;

  Output(this.to, this.kind, this.content, [DateTime? sentAt]) {
    this.sentAt = sentAt is DateTime ? sentAt : DateTime.now();
  }

  Future<Packet> encrypt(List<int> sk, int id) async {
    var key = await Crypto.deriveKey(sk, to);
    var from = Crypto.derivePK(sk);
    var data = Codec.encode([
      Utils.int64ToBinary(sentAt.microsecondsSinceEpoch),
      [kind],
      Codec.encode(content)
    ]);
    var cipher = await Crypto.encrypt(key, data);
    return Packet(to, from, cipher, id);
  }
}
