import 'dart:io';

class NetAddr {
  final int port;
  final String host;

  NetAddr(this.port, this.host);

  @override
  String toString() {
    return "$host:$port";
  }

  Future<List<InternetAddress>> lookup() async {
    return await InternetAddress.lookup(host);
  }

  Uri toUri(
      {required String? scheme,
      String? userInfo,
      Iterable<String>? pathSegments,
      Map<String, dynamic /*String|Iterable<String>*/ >? queryParameters}) {
    return Uri(
        scheme: scheme,
        host: host,
        port: port,
        userInfo: userInfo,
        pathSegments: pathSegments,
        queryParameters: queryParameters);
  }

  @override
  bool operator ==(Object other) =>
      other is NetAddr && other.host == other.host && other.port == other.port;

  static NetAddr fromString(String addr) {
    var lastColon = addr.lastIndexOf(':');
    if (lastColon < 0) {
      throw Exception("invalid net address $addr");
    }
    return NetAddr(
        int.parse(addr.substring(lastColon + 1)), addr.substring(0, lastColon));
  }

  @override
  int get hashCode => Object.hash(port, host);
}
