import 'codec.dart';
import 'utils.dart';

class Packet {
  static int genNowMsgID([int delta = 0]) {
    return ((DateTime.now().microsecondsSinceEpoch ~/ 1000) + delta) ~/ 10;
  }

  final List<int> to;
  late final int id;
  final List<int> from;
  final List<int> data;

  Packet(this.to, this.from, this.data, this.id);

  static Packet decode(List<int> b) {
    var parts = Codec.decode(b);
    return Packet(
        parts[0], parts[2], parts[3], Utils.parseBinaryToInt(parts[1]));
  }

  @override
  String toString() {
    return {
      "to": to,
      "from": from,
      "id": id,
      "data": data,
    }.toString();
  }

  List<int> encode() {
    if (id < 0) {
      return Codec.encode([
        to,
        [255, 255, 255, 255, 255, 255, 255, 255],
        from,
        data
      ]);
    }
    if (id == 0) {
      return Codec.encode(
          [to, Utils.int64ToBinary(Packet.genNowMsgID()), from, data]);
    }
    return Codec.encode([to, Utils.int64ToBinary(id), from, data]);
  }
}
