import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'crypto.dart';
import 'package:http/http.dart' as http;
import 'net_addr.dart';

var n = 0;

class Tail {
  final RawDatagramSocket _socket;
  late final Stream<Uint8List> stream;
  final NetAddr host;

  static Future<Tail> connect(NetAddr host) async {
    return Tail(await RawDatagramSocket.bind("0.0.0.0", 0), host);
  }

  Tail(this._socket, this.host) {
    stream = _socket
        .map((event) {
          return _socket.receive()?.data;
        })
        .where((event) => event != null)
        .map((event) => event!);
  }

  Future<void> check(List<int> last) async {
    try {
      _socket.send(last, (await host.lookup()).first, host.port);
    } catch (_) {}
  }

  void close() {
    return _socket.close();
  }

  static Future<void> push(
      NetAddr addr, List<int> data, List<int> sk, Duration timeout) async {
    var cli = http.Client();
    var timer = Timer(timeout, () {
      cli.close();
    });
    try {
      var resp = await cli.post(addr.toUri(scheme: "http"),
          headers: {
            "Content-Signature": base64UrlEncode(await Crypto.sign(sk, data))
          },
          body: data);
      if (resp.statusCode != 204) {
        throw Exception(resp.body.isEmpty ? resp.statusCode : resp.body);
      }
    } finally {
      cli.close();
      if (timer.isActive) {
        timer.cancel();
      }
    }
  }
}
