import 'dart:math';

import 'package:test/test.dart';
import 'package:yona/yona.dart';

void main() {
  test("deriveKey", () async {
    var sk1 = await Crypto.genSK();
    var sk2 = await Crypto.genSK();
    assert(sk2 != sk1);
    var pk1 = Crypto.derivePK(sk1);
    var pk2 = Crypto.derivePK(sk2);
    var k1 = await Crypto.deriveKey(sk1, pk2);
    var k2 = await Crypto.deriveKey(sk2, pk1);
    assert(Utils.equal(k1, k2));
  });
  test("encryptAndDecrypt", () async {
    var text = await Crypto.randBytes(9999);
    var key = await Crypto.randKey();
    var decoded = await Crypto.decrypt(key, await Crypto.encrypt(key, text));
    assert(Utils.equal(text, decoded));
  });
  test("randBytesLength", () async {
    var len = Random.secure().nextInt(500);
    var rand = await Crypto.randBytes(len);
    assert(rand.length == len);
  });
  test("SignAndVerify", () async {
    var sk = await Crypto.genSK();
    var pk = Crypto.derivePK(sk);
    var data = await Crypto.randBytes(100);
    var sig = await Crypto.sign(sk, data);
    assert(await Crypto.verify(data, sig, pk));
  });
}
