import 'dart:async';

import 'package:test/scaffolding.dart';
import 'package:yona/yona.dart';

void main() async {
  test("packet loss", () async {
    var node1 = await Node.create(
        await Crypto.genSK(), NetAddr.fromString("yonafediverse.com:7777"));
    await node1.publish("node1");
    var sent = 0;
    var set = Set<int>.identity();
    var completer = Completer();
    var finality = Timer(Duration(seconds: 30), () {
      completer.completeError(Exception("lost ${sent - set.length}"));
    });
    var timer = Timer.periodic(Duration(milliseconds: 100), (timer) async {
      await node1.send(Output(node1.pk(), 1, [await Crypto.randBytes(33)]));
      sent++;
    });
    node1.stream.forEach((packet) {
      set.add(packet.sentAt.millisecondsSinceEpoch);
      var lost = sent - set.length;
      if (!timer.isActive && lost == 0) {
        finality.cancel();
        completer.complete();
        node1.close();
      }
    });
    Timer(Duration(seconds: 5), () {
      timer.cancel();
    });
    await completer.future;
  });
}
